#pragma once

#include <fstream>
#include <iostream>
#include <string>
#include <ctime>
#include <deque>
#include <mutex>

enum class LogLevel {
    DEBUGL = 0,
    INFOL = 10,
    WARNINGL = 20,
    ERRORL = 30,
    CRITICALL = 40
};

class Logger {
public:
    static Logger& GetInstance() {
        static Logger instance;
        return instance;
    }

    Logger(const Logger&) = delete;
    Logger& operator=(const Logger&) = delete;

    void Initialize(const std::string &logFile = "", LogLevel consoleLevel = LogLevel::DEBUGL, LogLevel fileLevel = LogLevel::DEBUGL);
    void Log(LogLevel level, const std::string &message, const std::string &scope = "");
    void LogDebug(const std::string& message, const std::string& scope = "");
    void LogInfo(const std::string& message, const std::string& scope = "");
    void LogWarning(const std::string& message, const std::string& scope = "");
    void LogError(const std::string& message, const std::string& scope = "");
    void LogCritical(const std::string& message, const std::string& scope = "");

    void DumpLogs(const std::string& dumpFileName = "crash_dump.log");

private:
    Logger() = default;
    ~Logger();

    std::ofstream logFileStream;
    std::string logFileName;
    LogLevel consoleLogLevel;
    LogLevel fileLogLevel;
    std::mutex logMutex;

    std::deque<std::string> memoryLogs;
    size_t memoryLogSize = 100;

    std::string FormatMessage(LogLevel level, const std::string& message, const std::string& scope);
    std::string GetTimestamp();
    std::string LogLevelToString(LogLevel level);
    bool ShouldLogToConsole(LogLevel level);
    bool ShouldLogToFile(LogLevel level);
};
